@@ -803,7 +803,7 @@ static inline char *btrfs_err_str(enum btrfs_err_code err_code)
 		case BTRFS_ERROR_DEV_RAID1C4_MIN_NOT_MET:
 			return "unable to go below four devices on raid1c4";
 		case BTRFS_ERROR_DEV_RAID10_MIN_NOT_MET:
-			return "unable to go below four devices on raid10";
+			return "unable to go below four/two devices on raid10";
 		case BTRFS_ERROR_DEV_RAID5_MIN_NOT_MET:
 			return "unable to go below two devices on raid5";
 		case BTRFS_ERROR_DEV_RAID6_MIN_NOT_MET:
